/* ----------------------------------------------------------------------------------------------------
    Generic script to automate grid capabilities
    Author      : Sven Dickmans, Autodesk
    Imports     : -
    Last Update : Apr 3, 2019
  ---------------------------------------------------------------------------------------------------- */
    

/* ----------------------------------------------------------------------------------------------------
    Add current object with details to selected parent's grid
    Use updateRelatedGrid if pick list allows to select a single parent only, use updateRelatedGrids otherwise
    - fieldNew  : Pick list field that defines the parent whose grid needs to be updated
    - fieldOld  : Pick list field to store that last selected parent (to recognize changes)
   ---------------------------------------------------------------------------------------------------- */
function updateRelatedGrid(fieldNew, fieldOld) {
    
    var valueNew = item[fieldNew];
    var valueOld = item[fieldOld];
    
    if(valueNew !== valueOld) {
    
        addToGrid(valueNew);
        removeFromGrid(valueOld);
        item[fieldOld] = item[fieldNew];
    
    } else {
        
        updateExistingGrid(valueNew);
        
    }

}
function updateRelatedGrids(fieldNew, fieldOld) {
    
    var elementsNew = item[fieldNew];
    var elementsOld = item[fieldOld];
    
    var idsNew = getObjectIdsOfPickList(elementsNew);
    var idsOld = getObjectIdsOfPickList(elementsOld);
    var itemId = item.master.dmsID;
    
    if(idsNew !== idsOld) {
        
        for(var i = 0; i < elementsNew.length; i++) {
            
            var elementNew   = elementsNew[i];
            var elementIdNew = elementNew.master.dmsID;
            
            if(idsOld.indexOf(elementIdNew) == -1) { 
                
                addToGrid(elementNew);
                elementsOld.add(elementNew);
                
            } else {
                
                updateExistingGrid(elementNew);
                
            }
        
        }
        
        for(var j = elementsOld.length - 1; j >= 0; j--) {
            
            var elementOld = elementsOld[j];
            var elementIdOld = elementOld.master.dmsID;
            
            if(idsNew.indexOf(elementIdOld) == -1) { 
                
                removeFromGrid(elementOld);
                elementsOld.remove(elementOld);
                
            }
        
        }

    } else {
        
        for(var k = 0; k < elementsNew.length; k++) { updateExistingGrid(elementsNew[k]); }
        
    }
    
}
function updateExistingGrid(parent) {
    
    if(parent !== null) {
    
        var bAdd    = true;
        var grid    = parent.grid;
        
        for(var i = 0; i < grid.length; i++) {
            var row = grid[i];
            if(row.ITEM == item) {
                bAdd = false;
                syncGridRow(row, item);
            }
        }
        
        if(bAdd) addToGrid(parent);
    
    }
    
}
function addToGrid(parent) {

    if(parent !== null) {

        var grid = parent.grid;
        var row  = grid.addRow();
        
        syncGridRow(row, item);
        
    }
    
}
function syncGridRow(row, item) {
    
    var columns = Object.keys(row);
    var fields  = Object.keys(item);
    
    for(var i = 0; i < columns.length; i++) {
        
        var column = columns[i];
    
        if(column == "ITEM") { row.ITEM = item; }
        else if(column == "WORKFLOW_STATUS") { row.WORKFLOW_STATUS = item.descriptor.workflowState; }
        else {
            
            for(var j = 0; j < fields.length; j++) {
        
                var field = fields[j];
                if(field == column)  row[column] = item[field];
                
            }
            
        }
    
    }
    
}
function removeFromGrid(parent) {
    
   if(parent !== null) {
    
        var grid = parent.grid;
        
        for(var i = grid.length - 1; i >= 0; i--) {
            
            var row = grid[i];
            if(row.ITEM == item) row.remove();
            
        }
    
   }
    
}
function getObjectIdsOfPickList(elements) {
    
    var result = "";
    
    for(var i = 0; i < elements.length; i++) { result += elements[i].master.dmsID + ";"; }
    
    return result;
    
}


/* ----------------------------------------------------------------------------------------------------
    Clone existing grid from one item to another (e.g. for templates) 
    - source  : Item whose grid should be copied (e.g. item.TEMPLATE)
    - target  : Item where to copy the grid to (e.g. item)
   ---------------------------------------------------------------------------------------------------- */
function cloneGrid(source, target) {
    
    if(source === null) return;
    
    for(var i = 0; i < source.grid.length; i++) {
        
        var rowRef = source.grid[i];
        var rowNew = target.grid.addRow();
        
        var columnsRef = Object.keys(rowRef);
        var columnsNew = Object.keys(rowNew);
        
        for(var j = 0; j < columnsRef.length; j++) {
        
            var columnRef = columnsRef[j];
    
            for(var k = 0; k < columnsNew.length; k++) {
                
                var columnNew = columnsNew[k];
        
                if(columnRef == columnNew)  rowNew[columnNew] = rowRef[columnRef];
                
            }
            
        }
    
    }
    
}

