/* ----------------------------------------------------------------------------------------------------
    Utilities for tab Project Management
    Author      : Sven Dickmans, Autodesk
    Imports     : -
    Last Update : Apr 3, 2019
   ---------------------------------------------------------------------------------------------------- */


/* ----------------------------------------------------------------------------------------------------
    Create manual task in projects tab (will update existing tasks)
    - title      : Title of the manual task
    - start_date : Target start date of the task
    - end_date   : Target end date of the task
    - progress   : Current progress of the task (0-100)
   ---------------------------------------------------------------------------------------------------- */
function setProjectTask(title, start_date, end_date, progress) {
    
    var existing = false;
    var tasks = item.project.children;
    
    if(  end_date === null)   end_date = start_date;
    if(start_date === null) start_date = end_date;

    for(var i = 0; i < tasks.length; i++) {
        
        var taskTitle = tasks[i].title;
        if(taskTitle === title) {
            tasks[i].start_date = start_date;
            tasks[i].end_date = end_date;
            existing = true;
            break;
        }
        
    }
    
    if(!existing) item.project.addTask(title, start_date, end_date, progress);
    
}


/* ----------------------------------------------------------------------------------------------------
    Determine pending tasks count based on defined status
    - statusCompleted : Name of status that inidicates the task's completion
   ---------------------------------------------------------------------------------------------------- */
function checkForPendingTasks(statusCompleted) {

    var tasks = item.project.children;
    
    if(tasks === null) return false;
    
    for(var i = 0; i < tasks.length; i++) {
        
        if(tasks[i].status !== statusCompleted) return true;
        
    }
    
    return false;
    
}


/* ----------------------------------------------------------------------------------------------------
    Add / update / remove milestone based on define date
    - title     : Title of the milestone to create / update / remove
    - date      : date of the milestone
    - progress  : Progress of the milestone (e.g. 100)
   ---------------------------------------------------------------------------------------------------- */
function setProjectMilestone(title, date, progress) {
    
    var index   = -1;
    var project = item.project;
    
    for(var i = 0; i < project.children.length; i++) {
        if(title === project.children[i].title) {
            index = i;
            break;
        }
    }
    
    if(date !== null) {
        
        if(index >=0) {
            var task = project.children[index];
                task.progress = progress;
                task.start_date = date;
                task.end_date = date;
        } else {
            item.project.addTask(title, date, date, progress);
        }
        
    } else if(index >= 0) {
        
        project.children[index].remove();
        
    }
    
}


/* ----------------------------------------------------------------------------------------------------
    Add current item to project management tab of item selected in pick list field
    - fieldNew  : Pick list field in which users can select the parent (where the current item will be added)
    - fieldOld  : Pick list field to store previously selected parent (will be set by script automatically)
   ---------------------------------------------------------------------------------------------------- */
function updateRelatedProject(fieldNew, fieldOld) {
    
    var elemNew = item[fieldNew];
    var elemOld = item[fieldOld];
    
    
   /* if(fieldOld === null) {
        elemNew.project.addTask(item);  
        
    } else */
    
    if(elemNew !== elemOld) {

        if(elemNew !== null) elemNew.project.addTask(item); 
        
        if(elemOld !== null) {
        
            var elemTasks = elemOld.project.children;
            var itemId = item.master.dmsID;
            
            for(var i = elemTasks.length - 1; i >= 0; i--) {
                
                var elemTask = elemTasks[i];
                
                if(elemTask !== null) {
                    
                    if(elemTask.item !== null) {
                
                        var elemTaskId = elemTask.item.master.dmsID;
                        
                        if(elemTaskId == itemId) elemTask.remove(); 
                        
                    }
                    
                }
                
            }
        
        }
        
        item[fieldOld] = item[fieldNew];
        
    
    }

}
